// usb2lpt.c   V3.16.0
// THESE FUNCTIONS PERFORM THE IO TO THE USB2LPT 1.6 DEVICE
// based on V-USB driver by OBJECTIVE DEVELOPMENT Software GmbH
// see: http://www.obdev.at/products/vusb/index.html
// and  http://vusb.wikidot.com/
// and  the firmware usb2lpt6.c for USB2LPT 1.6 by Henrik Haftmann 
// see  http://www-user.tu-chemnitz.de/~heha/bastelecke/Rund%20um%20den%20PC/USB2LPT/index.html.en
// Pierre Vanhoucke / ON5GN

#if LIBUSB_PRESENT == 1

#ifndef u_int8_t
#ifdef uint8_t
#define u_int8_t uint8_t
#endif
#endif
#ifndef u_int16_t
#ifdef uint16_t
#define u_int16_t uint16_t
#endif
#endif
#ifndef u_int32_t
#ifdef uint32_t
#define u_int32_t uint32_t
#endif
#endif


#include <usb.h>  
#include <string.h>  
#include "globdef.h"
#include "uidef.h"
#include "screendef.h"

#define USBDEV_SHARED_VENDOR    0x16C0   // VOTI    VID
#define USBDEV_SHARED_PRODUCT   0x06B3   // USB2LPT PID 
#define USB_ERROR_NOTFOUND  1
#define USB_ERROR_ACCESS    2
#define USB_ERROR_IO        3
#define MAX_USB_ERR_CNT     6
#define DDR_OUT 0x00FF
int  usbGetStringAscii(usb_dev_handle *dev, int our_index, int langid, char *buf, int buflen);

usb_dev_handle *USB2LPT_handle;

void out_firmware (unsigned char port, unsigned char data) 
{ 
int nBytes;
unsigned char   in_byte[1];
int err_cnt; 
err_cnt =0;   
out_again:;    
nBytes = usb_control_msg
  (
  USB2LPT_handle, 
  USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, // request-type
  0x92,             //  request_id parameter
  port + data*256,  //  value parameter: first byte = portnumber, second byte = data
  0,                //  index parameter 
  (char *)in_byte,  //  pointer to  buffer
  sizeof(in_byte), 
  5000);            //  timeoutInMilliseconds
if (nBytes != 0)
  {
  err_cnt ++;
  if(err_cnt < MAX_USB_ERR_CNT)
    {
    lir_sleep(1000); // settling time
    goto out_again; 
    }
  else
    {
    printf("usb2lpt.c: Error during firmware setup \n"); 
    printf("usb error message: %s\n", usb_strerror());
    }
  }
}  


int usbOpenDevice(usb_dev_handle **device, int vendor, char *vendorName, 
                                               int product, char *productName)
{
struct usb_bus      *bus;
struct usb_device   *dev;
int                 errorCode = USB_ERROR_NOTFOUND;
static int          didUsbInit = 0;
char    string[256];
int     len;
if(!didUsbInit)
  {
  didUsbInit = 1;
  usb_init();
  }
usb_find_busses();
usb_find_devices();
for(bus=usb_get_busses(); bus; bus=bus->next)
  {
  for(dev=bus->devices; dev; dev=dev->next)
    {
    if(dev->descriptor.idVendor == vendor && 
                                         dev->descriptor.idProduct == product)
      {
// we need to open the device in order to query strings 
      USB2LPT_handle = usb_open(dev);       
      if(!USB2LPT_handle)
        {
        errorCode = USB_ERROR_ACCESS;
        printf("usb2lpt.c: Warning: cannot open USB device. \n");
        printf("usb error message:%s\n", usb_strerror());
        continue;
        }
// name does not matter if strings are NULL
      if(vendorName == NULL && productName == NULL) break;  
// now check whether the names match:
      len = usbGetStringAscii(USB2LPT_handle, 
              dev->descriptor.iManufacturer, 0x0409, string, sizeof(string));
      if(len < 0)
        {
        errorCode = USB_ERROR_IO;
        printf("usb2lpt.c: Warning: cannot query manufacturer for device.\n");
        printf("usb error message:%s\n", usb_strerror());
        }
      else
        {
        errorCode = USB_ERROR_NOTFOUND;
// printf("seen device from vendor ->%s<-\n", string); 
        if(strcmp(string, vendorName) == 0)
          {
          len = usbGetStringAscii(USB2LPT_handle, dev->descriptor.iProduct, 
                                                 0x0409, string, sizeof(string));
          if(len < 0)
            {
            errorCode = USB_ERROR_IO;
            printf("usb2lpt.c: Warning: cannot query product for device.\n");
            printf("usb error message:%s\n", usb_strerror());
            }
          else
            {
            errorCode = USB_ERROR_NOTFOUND;
           // printf("seen product ->%s<-\n", string); 
            if(strcmp(string, productName) == 0) break;
            }
          }
        }
      usb_close(USB2LPT_handle);
      USB2LPT_handle = NULL;
      }
    }
  if(USB2LPT_handle)  break;
  }
if(USB2LPT_handle != NULL)
  {
  errorCode = 0;
  *device = USB2LPT_handle;
  }
return errorCode;
}

int open_USB2LPT(void)
{
int retval, cnt;
char s[80];
char *usb2lpt_name_string="USB2LPT low-speed adapter";
if(usb2lpt_flag)return TRUE;
cnt=0;
clear_screen();
retry:;
retval=(usbOpenDevice(&USB2LPT_handle,
                      USBDEV_SHARED_VENDOR,
                      "haftmann#software",
                      USBDEV_SHARED_PRODUCT,
                      usb2lpt_name_string) );
if(retval != 0 )
  {
  sprintf(s,"%d Could not find \"%s\" with vid=0x%x pid=0x%x",retval, 
             usb2lpt_name_string, USBDEV_SHARED_VENDOR, USBDEV_SHARED_PRODUCT);
  lir_text(2,2,s);
  if(retval == USB_ERROR_NOTFOUND)
                           lir_text(2,4,"Is the USB2LPT device connected?");
#if OSNUM == OS_FLAG_WINDOWS
  if(retval == USB_ERROR_IO)lir_text(2,4,"USB permission denied");
#endif
#if OSNUM == OS_FLAG_LINUX
  if(retval == USB_ERROR_IO)lir_text(2,4,
                        "USB permission denied. Use sudo or run as root.");
#endif
  lir_text(2,6,"Press R to retry. Any other key to skip.");
  await_processed_keyboard();
  if(lir_inkey == 'R')
    {
    cnt++;
    sprintf(s,"Retry count %d",cnt);
    lir_text(2,8,s);
    goto retry;
    }
  clear_screen();  
  return FALSE;                 
  }
clear_screen();
lir_refresh_screen();
out_firmware(0x0F,0x00);	            	// Normal operation: Pullups on
out_firmware(0x0A,0x00);	             	// SPP-Modus
out_firmware(0x0D,(unsigned char)~DDR_OUT);     // Statusport      = Input  
out_firmware(0x0C,DDR_OUT);     	     	// Dataport        = Output
out_firmware(0x0E,DDR_OUT);	     	        // Controlport	   = Output
return TRUE;
}

void out_USB2LPT (unsigned char data, unsigned char port) 
{ 
int nBytes;
unsigned char   in_byte[1];
int err_cnt;
char s[80];
err_cnt =0;   
out_again:;       
nBytes = usb_control_msg
  (
  USB2LPT_handle, 
  USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, // request-type
  0x92,             //  request_id parameter
  port + data*256,  //  value parameter:  first byte = portnumber, second byte = data
  0,                //  index parameter 
  (char *)in_byte,  //  pointer to  buffer
  sizeof(in_byte), 
  5000);            //  timeoutInMilliseconds
if (nBytes != 0)
  {
  err_cnt ++;
  if(err_cnt < MAX_USB_ERR_CNT)
    {
    lir_sleep(1000); // settling time
    goto out_again; 
    }
  else
    {
    printf("usb2lpt.c: Error when writing to port=%i, data=%#4.2x, returncode=%i.\n",port,data,nBytes); 
    printf("usb error message: %s\n", usb_strerror());
    settextcolor(12);
    sprintf(s,"Error when writing to USB2LPT device: unable to control WSE converters  ");
    lir_text(15,screen_last_line,s);
    settextcolor(7);
    lir_refresh_screen();
    }
  }
//printf("                 OUT_USB  port= %i  data=%#4.2x \n",port,data); 
//fflush(stdout); 
return ;
} 

unsigned char in_USB2LPT( unsigned char port)
{    
int nBytes;
unsigned char   in_byte[1] ;           
int err_cnt;
char s[80];
err_cnt =0;
in_again:;
in_byte[0] = 0;
nBytes     = 0;
nBytes = usb_control_msg
  (
  USB2LPT_handle, 
  USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, // request-type
  0x01,   		// request_id parameter
  port, 		// value parameter
  0,      		// index parameter 
  (char *)in_byte,      // pointer to buffer
  sizeof(in_byte), 
  5000);                // timeoutInMilliseconds
if (nBytes!= 1)         // error recovery
  {
  err_cnt ++;
  if(err_cnt < MAX_USB_ERR_CNT)
    {
    lir_sleep(1000);  	// settling time
    goto in_again; 
    }
  else
    {
    printf("usb2lpt.c: Error when reading from port=%i, data=%#4.2x, returncode=%i.\n",port,in_byte[0],nBytes); 
    printf("usb error message: %s\n", usb_strerror());
    in_byte[0] = 0;
    settextcolor(12);
    sprintf(s,"Error when reading from USB2LPT device: unable to control WSE converters   ");
    lir_text(15,screen_last_line,s);
    settextcolor(7);
    lir_refresh_screen();
    }
  }
//printf("IN_USB  port= %i  data=%#4.2x \n",port,in_byte[0] );
//fflush(stdout); 
return in_byte[0] ;
}

#endif
